#!/usr/bin/env ruby
raise 'Please, use ruby1.8.4 or later.' if RUBY_VERSION < '1.8.4'
=begin

 Rantfile example for making a simple debian package of a ruby program
 using git-buildpackage with debuild

 Copyright (C) 2007 Ralph Amissah

 * License: LGPL - GNU Lesser General Public License
     [same license as Rant provided within the Rant package]

 * Ralph Amissah <ralph@amissah.com>
   Ralph Amissah <ralph.amissah@gmail.com>

 Rant is a Ruby 'ant' by Stefan Lang
 * Rant may be downloaded and installed from:
     http://make.rubyforge.org/

 Notes on use:

 This Rantfile is used in conjunction with the pkg.yml example provided
 (example is easily extended and works for package rant, and package diakonos)

 * package should be in directory with package name without package version
   number
 * Rantfile and pkg.yml should be in directory above package directory,
   e.g. for rant ls
     Rantfile, pkg.yml rant/

 * provide details of package in pkg.yml file [which is a yaml file] (see
   example provided)

 * once these two files are in place for help on package and on possible
   actions, enter:
     rant
     rant info
     rant -T

 * NOTE to create independent rant independent ruby script:
     rant-import --auto [provide name of ruby (rant exported) script]

=end
#% produce a makefile suitable for the target platform
#require 'mkmf'
#create_makefile("sisu")
module RC
  require 'date'
  require 'find'
  require 'fileutils'
  require 'rbconfig.rb'
  #require 'yaml'
  include FileUtils::Verbose
  class Package
    attr_reader :name,:aka,:name_up,:name_deb,:summary,:author,:author_email,:homepage,:maintainer,:maintainer_email,:version_base,:version_project,:version_deb,:version_deb_patch,:git_up,:git_deb,:reprepro_path,:reprepro_branch,:remote_host,:remote_user,:remote_archive,:remote_apt_archive,:gpg_mount,:ymd,:week_slash_day,:sys_arch,:sys_arch_site,:dir_bin,:dir_lib,:dir_data,:dir_conf,:dir_man,:dir_output,:rubylib,:pwd
    def initialize
      @pkg={}
      @dv='-'
      @home,@pwd,@user=ENV['HOME'],ENV['PWD'],ENV['USER']
      @git_up='upstream'
      @git_deb='master'
      @gpg_mount=nil
      yml=false
      path_up=@pwd.gsub(/\/[^\/]+$/,'')
      rc_yml_path=[@pwd,path_up]
      rc_yml=['pkgrc','package','pkg']
      rc_yml_path.each do |pkgrc_path|
        rc_yml.each do |pkgrc|
          pkg="#{pkgrc_path}/#{pkgrc}.yml"
          if FileTest.exist?(pkg)
            unless @rc
              require 'yaml'
              @rc ||= YAML::load(File::open(pkg))
              yml=true
              break
            end
          end
        end
      end
      @today=Date.today.to_s
      @time=Time.now
      day=if @time.day.to_s =~/^(\d)$/; "0#{$1}"
      else @time.day.to_s
      end
      month=if @time.month.to_s =~/^(\d)$/; "0#{$1}"
      else @time.month.to_s
      end
      @ymd="#{@time.year}-#{month}-#{day}"
      @w=@time.strftime("%W")
      @weekonly=@w
      @week="#{@time.year}w#@w"
      @monthonly=month
      @month="#{@time.year}#{@time.month}"
      @year=@time.year
      d=@time.wday
      @week_stamp="#{@time.year}w#@w"
      @week_slash_day="#@week_stamp/#{d}"
      @week_dash_day="#@week_stamp-#{d}"
                                                                 #%% system configuration
      @host=Config::CONFIG['host']
      @sys_arch=Config::CONFIG['archdir']
      @sys_arch_site=Config::CONFIG['sitearchdir']
      @dir_bin=Config::CONFIG['bindir']
      @dir_lib=Config::CONFIG['sitelibdir']
      @dir_data=Config::CONFIG['datadir']
      @dir_conf=Config::CONFIG['sysconfdir']                     #etc
      @dir_man=Config::CONFIG['mandir']
      @dir_output="#{Config::CONFIG['localstatedir']}/#{@pkg[:pn]}"
      @rubylib=Config::CONFIG['LIBRUBYARG_SHARED']
      @pwd=Dir.pwd
      if yml
        if defined? @rc['package']['name']
          @name=@rc['package']['name']
        else puts 'pkg.yml does not provide package name as required [package:name:]'
          exit
        end
        if defined? @rc['package']['version']
          @version_project=@rc['package']['version']
        else puts 'pkg.yml does not provide upstream package version number as required [package:version:]'
          exit
        end
        @summary=if defined? @rc['package']['summary']
          @rc['package']['summary']
        else ''
        end
        @author=if defined? @rc['package']['author']
          @rc['package']['author']
        else ''
        end
        @author_email=if defined? @rc['package']['email']
          @rc['package']['email']
        else ''
        end
        @homepage=if defined? @rc['package']['homepage']
          @rc['package']['homepage']
        else ''
        end
        if defined? @rc['package']['debian_patch']
          @version_deb_patch=@rc['package']['debian_patch']
        else puts 'pkg.yml does not provide debian patch level as required [package:debian_patch:]'
          exit
        end
        if defined? @rc['package']['alias']
          @aka=@rc['package']['alias']
        end
        if defined? @rc['git_build_package']['upstream']
          @git_up=@rc['git_build_package']['upstream']
        end
        if defined? @rc['git_build_package']['debian']
          @git_deb=@rc['git_build_package']['debian']
        end
        if defined? @rc['reprepro']['path']
          @reprepro_path=@rc['reprepro']['path']
          @reprepro_branch=if defined? @rc['reprepro']['branch']
            @rc['reprepro']['branch']
          else puts 'please provide debian branch for reprepro, using unstable'
            'unstable'
          end
        end
        if defined? @rc['remote']['host']
          @remote_host=@rc['remote']['host']
          @remote_user=if defined? @rc['remote']['user']
            @rc['remote']['user']
          else puts 'using current user as default user'
            `whoami`
          end
          if defined? @rc['remote']['archive']
            @remote_archive=@rc['remote']['archive']
          else puts 'information on the directory path in the remote archive has not been provided'
          end
          @remote_user_and_host="#{@remote_user}@#{remote_host}"
          @remote_apt_archive="#@remote_user_and_host:#{@remote_archive}"
        end
        if defined? @rc['gpg']['mount']
          @gpg_mount="mount #{@rc['gpg']['mount']}"
        else ''
        end
        @aka ||=@name
        @name_up="#@name-#@version_project"
        @name_deb="#@name_up-#@version_deb_patch"
        @version_deb="#@version_project-#@version_deb_patch"
        @version_base=/(\d+\.\d+)\.\d+/.match(@version_project)[1]
      elsif FileTest.exist?("#@pwd/version")
        project=`cat version`.strip         #--> sisu-0.48.9-1
        @name_up,@name,@version_up,@version_base,deb_ver=/((.+?)-((\d+\.\d+)\.\d+))-(\d+)/.match(project)[1..5]
        @name_deb=project
        @version_deb=@version_project + '-' + deb_ver
      end
      x=''
      @maintainer=if defined? @rc['maintainer']['name']
        @maintainer=@rc['maintainer']['name']
      else
        name=[ENV['GIT_AUTHOR_NAME'],ENV['GIT_COMMITTER_NAME']]
        name.each do |n|
          x=n if n=~/\S+/
          break if n=~/\S+/
        end
        if x=~/\S+/; @maintainer=x
        else puts "please maintainer's name in pkg.yaml or provide shell rc file with GIT_AUTHOR_NAME or GIT_COMMITTER_NAME"
        end
      end
      x=''
      if defined? @rc['maintainer']['email']
        @maintainer_email=@rc['maintainer']['email']
      else
        email=[ENV['GIT_AUTHOR_EMAIL'],ENV['GIT_COMMITTER_EMAIL']]
        email.each do |n|
          x=n if n=~/\S+/
          break if n=~/@/
        end
        if x=~/\S+/; @maintainer_email=x
        else puts "please maintainer's email in pkg.yaml or provide shell rc file with GIT_AUTHOR_EMAIL or GIT_COMMITTER_EMAIL"
        end
      end
    end
    def pkg_info
      aka="\n  :aka  - pkg alias:       #{@aka}" if @aka != @name
      puts <<WOK
  :name                    #@name#{aka}
  :name_up (upstream)      #@name_up
  :name_deb (debian)       #@name_deb
  :summary                 #@summary
  :author                  #@author
  :author_email            #@author_email
  :homepage                #@homepage
  :maintainer              #@maintainer
  :version_base            #@version_base
  :version_project         #@version_project
  :version_debian          #@version_deb

  :git_up                  #@git_up
  :git_deb                 #@git_deb

  #@maintainer  #@maintainer_email
WOK
    end
  end
end
